/*
	Copyright 2016 Michele "King_DuckZ" Santullo

	This file is part of CloonelJump.

	CloonelJump is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	CloonelJump is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CloonelJump.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef id7155F721B70C41B383896327C7D07E30
#define id7155F721B70C41B383896327C7D07E30

#if defined(VWR_OUTER_NAMESPACE)
#	define CLOONEL_OUTER_NAMESPACE_WAS_DEFINED
#	pragma push_macro("VWR_OUTER_NAMESPACE")
#	undef VWR_OUTER_NAMESPACE
#endif
#define VWR_OUTER_NAMESPACE cloonel

#include "cloonel_vectorwrapper/vectorwrapper.hpp"
#include "cloonel_vectorwrapper/vector_cast.hpp"
#include "compatibility.h"
#include <array>
#include <cstdint>
#if !defined(NDEBUG)
#	include <iostream>
#endif
#include <utility>

#define SPECIALIZE_ARRAY_VECTOR(TYPE, DIM) \
    template <> \
    struct VectorWrapperInfo<std::array<TYPE, DIM>> { \
        enum { dimensions = DIM }; \
        typedef TYPE scalar_type; \
        typedef std::array<scalar_type, dimensions> vector_type; \
        static scalar_type& get_at (uint32_t parIndex, vector_type& parVector) { \
            return parVector[parIndex]; \
        } \
    }

namespace cloonel {
namespace vwr {
	SPECIALIZE_ARRAY_VECTOR(float, 2);
	SPECIALIZE_ARRAY_VECTOR(uint16_t, 2);
#if !defined(NDEBUG)
	SPECIALIZE_ARRAY_VECTOR(int16_t, 2);
#endif
	SPECIALIZE_ARRAY_VECTOR(int32_t, 2);

#if !defined(NDEBUG)
	template <typename V>
	std::ostream& operator<< (std::ostream& parStream, const Vec<V>& parVec) {
		parStream << '<';
		for (uint32_t z = 0; z < Vec<V>::dimensions - 1; ++z) {
			std::cout << parVec[z] << ", ";
		}
		std::cout << parVec[Vec<V>::dimensions - 1] << '>';
		return parStream;
	}
#endif
} //namespace vwr
} //namespace cloonel

//make stuff from CloonelJump compile happily
namespace cloonel {
	template <typename T, uint32_t S>
	using Vector = vwr::Vec<std::array<T, std::size_t(S)>, S>;

	using ushort2 = Vector<uint16_t, 2>;
#if !defined(NDEBUG)
	using short2 = Vector<int16_t, 2>;
#endif
	using float2 = Vector<float, 2>;
	using int2 = Vector<int32_t, 2>;

	using vwr::vector_cast;
} //namespace cloonel

#undef SPECIALIZE_ARRAY_VECTOR

#if defined(CLOONEL_OUTER_NAMESPACE_WAS_DEFINED)
#	undef CLOONEL_OUTER_NAMESPACE_WAS_DEFINED
#	pragma pop_macro("VWR_OUTER_NAMESPACE")
#else
#	undef VWR_OUTER_NAMESPACE
#endif

#endif
